@@ -26,8 +26,8 @@
 #define MAXPRIOLEN 24
 
 struct lxc_handler;
-int lxc_rename_nsgroup(const char *name, struct lxc_handler *handler);
-int lxc_unlink_nsgroup(const char *name);
+int lxc_cgroup_create(const char *name, pid_t pid);
+int lxc_cgroup_destroy(const char *name);
 int lxc_cgroup_path_get(char **path, const char *name);
-
+int lxc_cgroup_nrtasks(const char *name);
 #endif
