@@ -31,6 +31,7 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/utsname.h>
+#include <sys/personality.h>
 #include <arpa/inet.h>
 #include <netinet/in.h>
 #include <net/if.h>
@@ -43,6 +44,7 @@
 
 lxc_log_define(lxc_confile, lxc);
 
+static int config_personality(const char *, char *, struct lxc_conf *);
 static int config_pts(const char *, char *, struct lxc_conf *);
 static int config_tty(const char *, char *, struct lxc_conf *);
 static int config_cgroup(const char *, char *, struct lxc_conf *);
@@ -61,6 +63,7 @@ static int config_network_hwaddr(const char *, char *, struct lxc_conf *);
 static int config_network_vlan_id(const char *, char *, struct lxc_conf *);
 static int config_network_mtu(const char *, char *, struct lxc_conf *);
 static int config_network_ipv4(const char *, char *, struct lxc_conf *);
+static int config_network_script(const char *, char *, struct lxc_conf *);
 static int config_network_ipv6(const char *, char *, struct lxc_conf *);
 static int config_cap_drop(const char *, char *, struct lxc_conf *);
 static int config_console(const char *, char *, struct lxc_conf *);
@@ -74,6 +77,7 @@ struct config {
 
 static struct config config[] = {
 
+	{ "lxc.arch",                 config_personality          },
 	{ "lxc.pts",                  config_pts                  },
 	{ "lxc.tty",                  config_tty                  },
 	{ "lxc.cgroup",               config_cgroup               },
@@ -88,6 +92,7 @@ static struct config config[] = {
 	{ "lxc.network.name",         config_network_name         },
 	{ "lxc.network.macvlan.mode", config_network_macvlan_mode },
 	{ "lxc.network.veth.pair",    config_network_veth_pair    },
+	{ "lxc.network.script.up",    config_network_script       },
 	{ "lxc.network.hwaddr",       config_network_hwaddr       },
 	{ "lxc.network.mtu",          config_network_mtu          },
 	{ "lxc.network.vlan.id",      config_network_vlan_id      },
@@ -418,9 +423,9 @@ static int config_network_ipv4(const char *key, char *value,
 	 * prefix and address
 	 */
 	if (!bcast) {
-		inetdev->bcast.s_addr =
-			htonl(INADDR_BROADCAST << (32 - inetdev->prefix));
-		inetdev->bcast.s_addr &= inetdev->addr.s_addr;
+		inetdev->bcast.s_addr = inetdev->addr.s_addr;
+		inetdev->bcast.s_addr |=
+			htonl(INADDR_BROADCAST >>  inetdev->prefix);
 	}
 
 	lxc_list_add(&netdev->ipv4, list);
@@ -475,6 +480,57 @@ static int config_network_ipv6(const char *key, char *value,
 	return 0;
 }
 
+static int config_network_script(const char *key, char *value,
+				 struct lxc_conf *lxc_conf)
+{
+	struct lxc_netdev *netdev;
+
+	netdev = network_netdev(key, value, &lxc_conf->network);
+	if (!netdev)
+	return -1;
+
+	char *copy = strdup(value);
+	if (!copy) {
+		SYSERROR("failed to dup string '%s'", value);
+		return -1;
+	}
+	if (strcmp(key, "lxc.network.script.up") == 0) {
+		netdev->upscript = copy;
+		return 0;
+	}
+	SYSERROR("Unknown key: %s", key);
+	free(copy);
+	return -1;
+}
+
+static int config_personality(const char *key, char *value,
+			      struct lxc_conf *lxc_conf)
+{
+	struct per_name {
+		char *name;
+		int per;
+	} pername[4] = {
+		{ "x86", PER_LINUX32 },
+		{ "i686", PER_LINUX32 },
+		{ "x86_64", PER_LINUX },
+		{ "amd64", PER_LINUX },
+	};
+
+	int i;
+
+	for (i = 0; i < sizeof(pername); i++) {
+
+		if (strcmp(pername[i].name, value))
+		    continue;
+
+		lxc_conf->personality = pername[i].per;
+		return 0;
+	}
+
+	ERROR("unsupported personality '%s'", value);
+	return -1;
+}
+
 static int config_pts(const char *key, char *value, struct lxc_conf *lxc_conf)
 {
 	int maxpts = atoi(value);
