@@ -47,7 +47,7 @@ extern int lxc_console(const char *name, int ttynum, int *fd)
 		.request = { .type = LXC_COMMAND_TTY, .data = ttynum },
 	};
 
-	ret = lxc_command(name, &command, &stopped);
+	ret = lxc_command_connected(name, &command, &stopped);
 	if (ret < 0 && stopped) {
 		ERROR("'%s' is stopped", name);
 		return -1;
