@@ -20,7 +20,9 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
+#define _GNU_SOURCE
 #include <stdio.h>
+#undef _GNU_SOURCE
 #include <stdlib.h>
 #include <unistd.h>
 #include <libgen.h>
@@ -32,8 +34,11 @@
 #include <sys/wait.h>
 #include <pwd.h>
 
-#include <lxc/log.h>
-#include <lxc/namespace.h>
+#include "caps.h"
+#include "log.h"
+#include "namespace.h"
+#include "cgroup.h"
+#include "error.h"
 
 lxc_log_define(lxc_unshare_ui, lxc);
 
@@ -154,6 +159,7 @@ int main(int argc, char *argv[])
 {
 	int opt, status;
 	int ret;
+	char *pid_name;
 	char *namespaces = NULL;
 	char **args;
 	int flags = 0;
@@ -180,6 +186,10 @@ int main(int argc, char *argv[])
 
 	args = &argv[optind];
 
+	ret = lxc_caps_init();
+	if (ret)
+		return ret;
+
         ret = lxc_fill_namespace_flags(namespaces, &flags);
  	if (ret)
 		usage(argv[0]);
@@ -200,6 +210,12 @@ int main(int argc, char *argv[])
 		return -1;
 	}
 
-	return status;
-}
+	if (asprintf(&pid_name, "%d", pid) == -1) {
+		ERROR("pid_name: failed to allocate memory");
+		return -1;
+	}
+	lxc_cgroup_destroy(pid_name);
+	free(pid_name);
 
+	return  lxc_error_set_and_log(pid, status);
+}
