@@ -21,12 +21,11 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
-struct epoll_event;
+#include <list.h>
 
 struct lxc_epoll_descr {
 	int epfd;
-	int nfds;
-	struct epoll_event *ev;
+	struct lxc_list handlers;
 };
 
 typedef int (*lxc_mainloop_callback_t)(int fd, void *data, 
@@ -40,6 +39,6 @@ extern int lxc_mainloop_add_handler(struct lxc_epoll_descr *descr, int fd,
 
 extern int lxc_mainloop_del_handler(struct lxc_epoll_descr *descr, int fd);
 
-extern int lxc_mainloop_open(int size, struct lxc_epoll_descr *descr);
+extern int lxc_mainloop_open(struct lxc_epoll_descr *descr);
 
 extern int lxc_mainloop_close(struct lxc_epoll_descr *descr);
